"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const mobile_devices_controller_1 = require("mobile-devices-controller");
class DeviceManager {
    constructor() {
    }
    startDevice(args) {
        return __awaiter(this, void 0, void 0, function* () {
            args.appiumCaps.platformName = args.appiumCaps.platformName.toLowerCase();
            let device = DeviceManager.getDefaultDevice(args);
            if (process.env["DEVICE_TOKEN"]) {
                device.token = process.env["DEVICE_TOKEN"];
                device.name = process.env["DEVICE_NAME"] || device.name;
                const allDevices = yield mobile_devices_controller_1.DeviceController.getDevices({ platform: device.platform });
                const foundDevice = mobile_devices_controller_1.DeviceController.filter(allDevices, { token: device.token.replace("emulator-", "") })[0];
                utils_1.logInfo("Device: ", foundDevice);
                return foundDevice;
            }
            // When isSauceLab specified we simply do nothing;
            if (args.isSauceLab || args.ignoreDeviceController) {
                args.ignoreDeviceController = true;
                DeviceManager._emulators.set(args.runType, device);
                return device;
            }
            const allDevices = yield mobile_devices_controller_1.DeviceController.getDevices({ platform: args.appiumCaps.platformName });
            if (!allDevices || allDevices === null || allDevices.length === 0) {
                utils_1.logError("We couldn't find any devices. We will try to proceed to appium!");
                console.log("Available devices:\n", allDevices);
                utils_1.logWarn(`We couldn't find any devices. We will try to proceed to appium!`);
                if (args.appiumCaps.platformVersion.toLowerCase() === mobile_devices_controller_1.Platform.ANDROID) {
                    const errMsg = `1. Check if ANDROID_HOME environment variable is set correctly!\n
                2. Check if avd manager is available!
                3. Check appium capabilites and provide correct device options!`;
                    utils_1.logWarn(errMsg);
                }
                args.ignoreDeviceController = true;
            }
            const searchObj = args.appiumCaps.udid ? { token: args.appiumCaps.udid } : { name: args.appiumCaps.deviceName, apiLevel: args.appiumCaps.platformVersion };
            let searchedDevices = mobile_devices_controller_1.DeviceController.filter(allDevices, searchObj);
            if (!searchedDevices || searchedDevices.length === 0) {
                utils_1.logError(`No such device ${args.appiumCaps.deviceName}!!!`);
                utils_1.logWarn("All properties like platformVersion, deviceName etc should match!");
                utils_1.logInfo("Available devices:\t\t\t\t");
                console.log('', allDevices);
            }
            if (searchedDevices && searchedDevices.length > 0) {
                // Should find new device
                if (!args.reuseDevice) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.SHUTDOWN })[0];
                }
                // If there is no shutdown device
                if (!device || device === null || !device.status) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.BOOTED })[0];
                }
                // In case reuse device is true but there weren't any booted devices. We need to fall back and boot new one.
                if (!device || device === null && args.reuseDevice) {
                    device = mobile_devices_controller_1.DeviceController.filter(searchedDevices, { status: mobile_devices_controller_1.Status.SHUTDOWN })[0];
                }
                if (device.status === mobile_devices_controller_1.Status.SHUTDOWN) {
                    yield mobile_devices_controller_1.DeviceController.startDevice(device);
                    utils_1.logInfo("Started device: ", device);
                }
                else {
                    device.type === mobile_devices_controller_1.DeviceType.DEVICE ? utils_1.logInfo("Device is connected:", device) : utils_1.logInfo("Device is already started", device);
                    if (!args.reuseDevice && device.type !== mobile_devices_controller_1.DeviceType.DEVICE) {
                        utils_1.logInfo("Option --reuseDevice is set to false, the device would be shut down and restart!");
                        utils_1.logInfo("Use --reuseDevice to preserve device state!");
                        mobile_devices_controller_1.DeviceController.kill(device);
                        yield mobile_devices_controller_1.DeviceController.startDevice(device);
                    }
                }
            }
            if (device.platform === mobile_devices_controller_1.Platform.ANDROID) {
                mobile_devices_controller_1.AndroidController.clearLog(device);
            }
            DeviceManager._emulators.set(args.runType, device);
            if (!device || !device.token) {
                console.error("Check appium capabilites and provide correct device options!");
                process.exit(1);
            }
            return device;
        });
    }
    stopDevice(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (DeviceManager._emulators.has(args.runType)
                && !args.reuseDevice
                && !args.isSauceLab
                && !args.ignoreDeviceController) {
                const device = DeviceManager._emulators.get(args.runType);
                yield DeviceManager.kill(device);
            }
        });
    }
    installApp(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (args.isIOS) {
                mobile_devices_controller_1.IOSController.installApp(args.device, args.appiumCaps.app);
                console.log(`Application is successfully installed!`);
            }
            else {
                mobile_devices_controller_1.AndroidController.installApp(args.device, args.appiumCaps.app);
            }
        });
    }
    uninstallApp(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (args.isIOS) {
                yield mobile_devices_controller_1.IOSController.uninstallApp(args.device, args.appPath, args.appiumCaps["bundleId"]);
            }
            else {
                yield Promise.resolve(mobile_devices_controller_1.AndroidController.uninstallApp(args.device, args.appiumCaps["appPackage"]));
            }
        });
    }
    static kill(device) {
        return __awaiter(this, void 0, void 0, function* () {
            yield mobile_devices_controller_1.DeviceController.kill(device);
        });
    }
    static getDefaultDevice(args, deviceName, token, type, platformVersion) {
        let device = new mobile_devices_controller_1.Device(deviceName || args.appiumCaps.deviceName, platformVersion || args.appiumCaps.platformVersion, type, args.appiumCaps.platformName.toLowerCase(), token, undefined, undefined);
        device.config = { "density": args.appiumCaps.density, "offsetPixels": args.appiumCaps.offsetPixels };
        delete args.appiumCaps.density;
        delete args.appiumCaps.offsetPixels;
        return device;
    }
    static setDontKeepActivities(args, driver, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const status = value ? 1 : 0;
            try {
                if (args.isAndroid) {
                    if (!args.ignoreDeviceController) {
                        mobile_devices_controller_1.AndroidController.setDontKeepActivities(value, args.device);
                    }
                    else if (args.relaxedSecurity) {
                        const output = yield DeviceManager.executeShellCommand(driver, { command: "settings", args: ['put', 'global', 'always_finish_activities', status] });
                        console.log(`Output from setting always_finish_activities to ${status}: ${output}`);
                        //check if set 
                        const check = yield DeviceManager.executeShellCommand(driver, { command: "settings", args: ['get', 'global', 'always_finish_activities'] });
                        console.info(`Check if always_finish_activities is set correctly: ${check}`);
                    }
                }
                else {
                    // Do nothing for iOS ...
                }
            }
            catch (error) {
                utils_1.logError(`Could not set don't keep activities: ${status}!`);
                utils_1.logError(error);
            }
        });
    }
    static executeShellCommand(driver, commandAndargs) {
        return __awaiter(this, void 0, void 0, function* () {
            const output = yield driver.execute("mobile: shell", commandAndargs);
            return output;
        });
    }
    static getDensity(args, driver) {
        return __awaiter(this, void 0, void 0, function* () {
            args.device.config = args.device.config || {};
            if (args.appiumCaps.platformName.toLowerCase() === "android") {
                if (!args.ignoreDeviceController) {
                    args.device.config.density = yield mobile_devices_controller_1.AndroidController.getPhysicalDensity(args.device);
                }
                if (args.relaxedSecurity && !args.device.config.density) {
                    const d = yield DeviceManager.executeShellCommand(driver, { command: "wm", args: ["density"] });
                    args.device.config.density = /\d+/ig.test(d) ? parseInt(/\d+/ig.exec(d)[0]) / 100 : NaN;
                    console.log(`Device density recieved from adb shell command ${args.device.config.density}`);
                }
                if (args.device.config.density) {
                    args.device.config['offsetPixels'] = mobile_devices_controller_1.AndroidController.calculateScreenOffset(args.device.config.density);
                }
            }
            else {
                mobile_devices_controller_1.IOSController.getDevicesScreenInfo().forEach((v, k, m) => {
                    if (args.device.name.includes(k)) {
                        args.device.config = {
                            density: args.device.config['density'] || v.density,
                            offsetPixels: v.actionBarHeight
                        };
                    }
                });
            }
        });
    }
    static applyDeviceAdditionsSettings(driver, args, sessionIfno) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args.device.config || !args.device.config.offsetPixels) {
                args.device.config = {};
                let density;
                if (sessionIfno && sessionIfno.length >= 1) {
                    density = sessionIfno[1].deviceScreenDensity ? sessionIfno[1].deviceScreenDensity / 100 : undefined;
                }
                if (density) {
                    console.log(`Get density from appium session: ${density}`);
                    args.device.config['density'] = density;
                    args.device.config['offsetPixels'] = mobile_devices_controller_1.AndroidController.calculateScreenOffset(args.device.config.density);
                }
                if (!density) {
                    yield DeviceManager.getDensity(args, driver);
                    density = args.device.config.density;
                    args.device.config['offsetPixels'] = mobile_devices_controller_1.AndroidController.calculateScreenOffset(args.device.config.density);
                }
                density ? utils_1.logInfo(`Device setting:`, args.device.config) : console.log(`Could not resolve device density. Please provide offset in appium config`);
            }
        });
    }
    getPackageId(device, appPath) {
        const appActivity = (device.type === mobile_devices_controller_1.DeviceType.EMULATOR || device.platform === mobile_devices_controller_1.Platform.ANDROID) ? mobile_devices_controller_1.AndroidController.getPackageId(appPath) : mobile_devices_controller_1.IOSController.getIOSPackageId(device.type, appPath);
        return appActivity;
    }
}
DeviceManager._emulators = new Map();
exports.DeviceManager = DeviceManager;
//# sourceMappingURL=device-manager.js.map